--[[
This script gathers panorama data : camera positions, names of views, image files, 
and saves them all to a json file. This can later be used to construct networked panoramas with a web app.

The GUI lists all the panoramic cameras in the scene, and allows the user to assign an image file for each camera view.
Finally there is a save button that saves the data: camera name, position, and image file path to a json file.
]]

json = require("json/json"); -- libary that will convert the lua table to json.

--
local settings =
{
    ["projectPath"]     = octane.project.getCurrentProject(), -- path to project
    ["projectName"]     = octane.file.getFileNameWithoutExtension(octane.project.getCurrentProject()), -- project name
    ["renderTargets"]   = octane.project.getSceneGraph():findNodes(octane.NT_RENDERTARGET), -- Gather all the render target nodes in the scene into an array.
    ["outputDirectory"] = nil,
}
--print("projectName: ", settings.projectName )

------------------------------------------------------ Project Name Text Editor
projectNameTE = octane.gui.createTextEditor({
    text   = settings.projectName,
    height = 20});

projectGroup = octane.gui.create -- Group to outline and contain Project Name field
{
     type     = octane.gui.componentType.GROUP,
     text     = "Panoscape Name",
     rows     = 1,
     cols     = 1,
     children =   { projectNameTE },
     height = 30,
     width = 200,
     padding  = { 5 },
     border   = false,
}

------------------------------------------------------ panoCams Array
-- Make an array of all panoramic cameras and their render node names.
panoCams = {};
renderTarget = {}; -- contains just the render targets that contain panoramic cameras.

-- Filter out the render targets with pano cameras from settings.renderTargets.
local pc = 1;
for i, entry in ipairs(settings.renderTargets) do
   -- print(entry)
    cam = octane.node.getConnectedNodeIx( entry, 1 );
   -- print(cam,  '/', cam.nodeType, '/', cam.name )
    if cam.nodeType == octane.NT_CAM_PANORAMIC then -- CAM_PANORAMIC
        panoCams[pc] = cam;
        renderTarget[pc] = entry;      
        --print(pc, panoCams[pc], entry.name);
        pc = pc + 1;
    end
end

--print('panoCams length 1:', #panoCams );

------------------------------------------------------ functions for creating gui elements
local function createLabel(text, pWidth)
    return octane.gui.create
    {
        type   = octane.gui.componentType.LABEL,    -- type of component
        text   = text,                              -- text that appears on the label
        width  = pWidth or 100,                               -- width of the label in pixels
        height = 24,                                -- height of the label in pixels
    }
end

-- helper to pop-up an error dialog and optionally halts the script
local function showError(text, halt)
    octane.gui.showDialog
    {
        type  = octane.gui.dialogType.ERROR_DIALOG,
        title = "Turntable Animation Error",
        text  = text,
    }
    if halt then error("ERROR: "..text) end
end

------------------------------------------------------- GUI header row for pano cameras
guiElements = {}; -- contains all gui elements that will be added to gui group 'panoGroup'
panoNameTextEditor = {}; -- Name of each panorama. Default is render target name
imageFileLabels = {}; -- gui labels that contain the path and file name for image files (should be the rendered image from a render target. jpg or png)
camPositions = {}; -- position for each camera
cells = 6; -- number of gui elements in a row
row = 1; -- counter for each pano

-- Top level Labels (the header row for the table of gui elements)
guiElements[1] =  createLabel("Index");
guiElements[2] =  createLabel("Render Target ");
guiElements[3] =  createLabel("See in Viewport");
guiElements[4] =  createLabel("Pano Name");
guiElements[5] =  octane.gui.create -- Import All Button - all files in folder set to all image file text fields. 
    {
        type     = octane.gui.componentType.BUTTON,
        name     = tostring(row),
        text     = "Browse Images All",
        width    = 120,
        height   = 20,
        callback = 
        -- Select the render target
        function()    
            --print('Import all image files');
            -- ask the user for an output directory for the results
			local ret = octane.gui.showDialog
			{
				type            = octane.gui.dialogType.FILE_DIALOG,
				title           = "Select output directory for the render results", 
				path            = octane.file.getParentDirectory(settings.projectPath),
                multiFile       = true,
				browseDirectory = false,
				save            = false,
			}
		
			if not ret.multiResult or #ret.multiResult == 0 then
				error("no image files imported") 
            else
                -- print('image files found: ', #ret.multiResult);
               -- loop through the image files found and set them into the imageFileLabels
                for index, value in ipairs(imageFileLabels) do
                    -- print(index, ret.multiResult[index]);
                    octane.gui.updateProperties(imageFileLabels[index], {text = ret.multiResult[index]});
                end
			end
        end
    }
guiElements[6] =  createLabel("Image File Path");

------------------------------------------------------ GUI elements for pano cameras
-- Creates Labels and Buttons inside cells 
-- print('panoCams length 2:', #panoCams );
for index, value in ipairs(panoCams) do
    local cellIndex = (index) * cells; -- Index
   -- print(index,  cellIndex, ':', panoCams[index], ':', renderTarget[index].name);
    local camIndex       = createLabel( tostring(index), 10 );    --                       Label Index
    local renderTargetLabel       = createLabel(renderTarget[index].name); --              Label Render Target
    imageFileLabels[index]       = createLabel("path/to/image "..tostring(index), 800); -- Image File Path
    -- print('render target: ', cellIndex, renderTarget[row].name); 
    local renderTargetButton = octane.gui.create --                                        Button Render Target
    { --                                                            button that renders this camera in viewport
        type     = octane.gui.componentType.BUTTON,
        name     = tostring(row),
        text     = "Render "..renderTarget[index].name,
        width    = 80,
        height   = 20,
        callback = 
        -- Select the render target
        function(component, event)    
            -- local id = tonumber(octane.gui.getProperties(component).name);
            local id = index;
            print('callback id: ', id); 
            print('select render target: ', renderTarget[id].name);
            octane.project.clearSelection();
            octane.project.select(renderTarget[id]);
            octane.render.callbackStop();
            octane.render.start {    renderTargetNode = renderTarget[id] };
            return function() 
                print('select render target: ', renderTarget[id].name);
            end
        end
    }

    ------------------------------------------------------ Text Editor Pano Name
    panoNameTextEditor[row] = octane.gui.createTextEditor({
            text    = renderTarget[index].name,
            height  = 20
        })

    ------------------------------------------------------ Button Browse Image
     -- loadImageButton - create a button to show choose file name and path
    local loadImageButton = octane.gui.create --                                          Button Browse Image
    {
        type     = octane.gui.componentType.BUTTON,
        text     = "Browse Image",
        name     = tostring(row),
        width    = 120,
        height   = 20,
        callback = 
        -- Select image file path and place it inside the imageFile Label
        function(component, event)    
            local id = tonumber(octane.gui.getProperties(component).name);
            print('load button component: ', component); 
          
            -- ask the user for an output directory for the results
			local ret = octane.gui.showDialog
			{
				type            = octane.gui.dialogType.FILE_DIALOG,
				title           = "Select output directory for the render results", 
				path            = octane.file.getParentDirectory(settings.projectPath),
				browseDirectory = false,
				save            = false,
			}
		
			if not ret.result or ret.result == "" then
				error("no output directory selected") 
            else
                print(ret.result);
                imageFileLabels[index].text = octane.gui.updateProperties(imageFileLabels[id], {text=ret.result})
			end
        end
    }
    
    ---------------------------------- place the newly created gui items inside the guiElements array for gui group.
    guiElements[cellIndex+1] =  camIndex;
    guiElements[cellIndex+2] =  renderTargetLabel;
    guiElements[cellIndex+3] =  renderTargetButton;
    guiElements[cellIndex+4] =  panoNameTextEditor[index];-- 
    guiElements[cellIndex+5] =  loadImageButton; --          Button Browse Image
    guiElements[cellIndex+6] =  imageFileLabels[index]; --   Image File Label

    ------------------------------------------------------ Camera Postions
   -- print('render target type:', type(renderTarget[index]));
    camera = octane.node.getConnectedNodeIx( renderTarget[index], 1 );
    --print('---- camera: ', c, camera, #camProps);
    pos = octane.node.getConnectedNodeIx(camera, 9);
    --print('pos', pos);
    position = pos:getAttribute(octane.A_VALUE);
    --print('position: ', position[1],  position[2],  position[3]);
    camPositions[row] = position;
      
    row = row + 1;

end

print( 'guiElements', #guiElements, row-1 );
print('cells: ', cells);
--print('guiElements: ', guiElements);
--print('camPositions: ', camPositions);

------------------------------------------------------------ gui Group Panorama Camera
panoGroup = octane.gui.create
{
     type     = octane.gui.componentType.GROUP,
     text     = "Panorama Cameras",
     rows     = row,
     cols     = cells,
     children =    guiElements ,
     width = 1200,
     padding  = { 5 },
     border   = true,
}

-----------------------------
-- Savce & Cancel buttons
 
------------------------------------------------------------ save button
local saveButton = octane.gui.create
{
    type   = octane.gui.componentType.BUTTON,
    text   = "Save",
    width  = 80,
    height = 20,
    callback = 
        function()
            print("save json");

            ----------------------- panoData 
            -- Create the table that will be used to make the json file
            panoData = {};
            for i,v in ipairs(camPositions) do  
                panoData[i] = 
                {
                    panoscapename = settings.projectName,
                    name = panoNameTextEditor[i].text,
                    x  =  v[1],
                    y  =  v[2],
                    z  =  v[3],
                    id = i-1,
                    index = i-1,
                    indexview = i,
                    path = imageFileLabels[i].text,
                    texture = octane.file.getFileName(imageFileLabels[i].text)
                }
                
            end 
            -- print('panoData length: ', #panoData);
            pdJson = json.encode(panoData);
            -- print(pdJson);

            -- Get the path to save the file to disk
            fileToSave = fileEditor.text;
            -- print( 'fileToSave', fileToSave );

           -- save to disk
            local file,err = io.open(fileToSave,'w')
            if file then
                -- print('file: ', file);
                file:write(tostring(pdJson));
                file:close();
            else
                print("error:", err) -- not so hard?
            end
        end
}
-------------------------------------------------------------- cancel button
local cancelButton = octane.gui.create
{
    type   = octane.gui.componentType.BUTTON,
    text   = "Cancel",
    width  = 80,
    height = 20,
    x = 740,
    callback = 
        function()
            octane.render.callbackStop();
        end
}
-------------------------------------------------------------- json file path text editor
-- file output
fileEditor = octane.gui.create -- creates an editor that will show the chosen file path
{
    type    = octane.gui.componentType.TEXT_EDITOR,
    text    = "",
    x       = 20,
    width   = 800,
    height  = 20,
    enable  = false,   
}
--------------------------------------------------------------- json file browse button
-- create a button to show a file chooser
fileChooseButton = octane.gui.create
{
    type     = octane.gui.componentType.BUTTON,
    text     = "Output...",
    width    = 80,
    height   = 20,
    callback = function()
        	-- ask the user for an output directory for the results
			local ret = octane.gui.showDialog
			{
				type            = octane.gui.dialogType.FILE_DIALOG,
				title           = "Select output directory for the render results", 
				path            = octane.file.getParentDirectory(settings.projectPath),
				browseDirectory = false,
				save            = true,
			}
		
			if not ret.result or ret.result == "" then
				error("no output directory selected")
			end
		
			settings.outputDirectory = ret.result
			fileEditor.text  = settings.outputDirectory
    end
}
---------------------------------------------------------------- file group - for save, cancel buttons
local fileGrp = octane.gui.create
{
    type     = octane.gui.componentType.GROUP,
    text     = "Output",
    rows     = 2,
    cols     = 2,
    children =
    {
        fileChooseButton, fileEditor, saveButton, cancelButton 
    },
    padding  = { 2 },
    inset    = { 5 },
}
---------------------------------------------------------------- panelStack
-- stack that puts all in place
local panelStack = octane.gui.create
{
    type     = octane.gui.componentType.PANEL_STACK,
    width    = 1600,
    height   = 720,
    children = { projectGroup, panoGroup,fileGrp}, 
    open     = { true, true, true },                     -- component open or closed
    captions = { "Panoscape Name",
                 "Assign image files for each pano camera",
                 "Save data to json file"} -- captions to appear above the children
}
---------------------------------------------------------------- window
-- window that holds all components
local window = octane.gui.create
{
    type     = octane.gui.componentType.WINDOW,
    text     = "panorama cameras",
    children = { panelStack },
    width    = panelStack:getProperties().width,
    height   = panelStack:getProperties().height,
}

window:showWindow()


